/*
** FILE NAME:	diir1.c
**
** PROJECT ID:	94-0040
**
** TITLE:	DIir1
**
** AUTHOR:	Ensigma Ltd.
**
** DESCRIPTION:	Perform infinite impulse response (IIR) filtering
**		with coefficients in double precision for one input sample
**
**		int DIir1(output, input, coeff, no_sections, workspace)
**
** PARAMETERS:
**              short output[]        : output samples yK-1.
**              short input           : input samples x.
**              const long coeff[]    : filter coefficients.
**                                     _____________________________________
**                                     | a0k  | a1k | a2k | b0k | b1k | b2k|
**                                     -------------------------------------
**              long no_sections      : number of second order filter sections K
**              long  *workspace      : variable containing pointer to filter
**                                         memory.
**
** RETURNS:     EDSP_OK               : success
**              EDSP_BAD_ARG          : no_sections < 1
**                                      a0k         < 0
**                                      a0k         > 32, k<K-1
**                                      a0k         > 48, k=K-1
**
** MODIFICATION HISTORY:
**              1.0  15-9-95    Initial revision
*/

#include <math.h>
#include <ensigdsp.h>

#define TWO_31		2147483648.0
#define	TWO_15		32768.0
#define	TWO_16		65536.0
#define LONG_MAX	TWO_31-1
#define LONG_MIN	-TWO_31
#define SHT_MAX		TWO_15-1
#define SHT_MIN		-TWO_15

int 
DIir1(short *output,
      short input,
      const long coeff[],
      long no_sections,
      long *workspace)
{
    /* Variable declaration */
    int k, i;
    double yn, dn;
    double x, div;

    /* Error Checking */
    if (no_sections < 1)
	return EDSP_BAD_ARG;

    /* Error checking for a0k   */
    for (k = 0; k < (no_sections - 1); k++)
	if ((coeff[6 * k] > 32) || (coeff[6 * k] < 0))
	    return EDSP_BAD_ARG;

    /* Error checking for a0K-1  */
    if ((coeff[6 * (no_sections - 1)] > 48) || (coeff[6 * (no_sections - 1)] < 0))
	return EDSP_BAD_ARG;

    x = input * TWO_16;		/* Shift input */

    for (k = 0; k < no_sections; k++)
    {
	/* d(n) = [a2*d(n-2) + a1*d(n-1) + x(n)<<31]/(1<<31) */
	dn = (double)coeff[6 * k + 2] * workspace[k * no_sections + 1] +
	    (double)coeff[6 * k + 1] * workspace[k * no_sections] +
	    x * TWO_31;

	/* Rounding */
	dn = floor(dn / TWO_31 + 0.5);

	/* Saturation */
	if (dn > LONG_MAX)
	    dn = LONG_MAX;
	else if (dn < LONG_MIN)
	    dn = LONG_MIN;

	/* y(n) = b2*d(n-2) + b1*d(n-1) + b0*d(n)  */
	yn = (double)coeff[6 * k + 5] * workspace[k * no_sections + 1] +
	    (double)coeff[6 * k + 4] * workspace[k * no_sections] +
	    coeff[6 * k + 3] * dn;

	/* d(n-2) = d(n-1) */
	workspace[k * no_sections + 1] = workspace[k * no_sections];

	/* d(n-1) = d(n) */
	workspace[k * no_sections] = dn;

	/* y(n) = [y(n)] >> a0k  */
	for (div = 1., i = 0; i < coeff[6 * k]; i++)
	    div *= 2.;

	/* Rounding */
	yn = floor(yn / div + 0.5);

	/* Saturation */
	if (yn > LONG_MAX)
	    yn = LONG_MAX;
	else if (yn < LONG_MIN)
	    yn = LONG_MIN;

	x = yn;			/* new Input */

    }
    /* Saturation */
    if (yn > SHT_MAX)
	yn = SHT_MAX;
    else if (yn < SHT_MIN)
	yn = SHT_MIN;
    *output = yn;		/* Output extraction */

    return EDSP_OK;		/* Return Success */
}
