/*
** FILE NAME:   fftreal.c
**
** PROJECT ID:  94-0040
**
** TITLE:       FftReal
**
** AUTHOR:      Ensigma Ltd.
**
** DESCRIPTION: This file contains the wrapper used to invoke a forward,
**              not-in-place, real FFT.
**
**              int FftReal(output, input, size, scale)
**
** PARAMETERS:
**              short output[]      : Complex output data
**              const short input[] : Complex input data
**              long size           : size of FFT
**              long scale          : scaling specification
**                                      No scaling = 0
**                                      Complete scaling = 0xffffffff
**
** RETURNS :    EDSP_OK       : success
**              EDSP_BAD_ARG  : size < 2
**                              size not a power of 2.
**                              size > max_fft_size
** MODIFICATION HISTORY:
**              1.0  20-7-95    Initial revision
*/

#include <ensigdsp.h>

int 
FftReal(short output[], const short input[], long size, long scale)
{
    /* Variable declaration */
    int i, index;
    long h1r, h1i, h2r, h2i;
    int test;

    /* Input Argument error testing */
    if ((size < 8) || (size > max_fft_size))
	return EDSP_BAD_ARG;

    /* test for power of 2 */
    for (test = size; test > 2; test /= 2)
	if (test & 1)
	    return EDSP_BAD_ARG;

    index = max_fft_size / size;/* Twiddle Factor index scaling */
    if ((i = FftComplex(output, input, (size >> 1), scale)) != EDSP_OK)
	return EDSP_BAD_ARG;

    /* Main Loop */
    for (i = 1; i <= (size >> 2); i++)
    {
	h1r = ((long)output[2 * i] + output[size - 2 * i]);
	h1i = ((long)output[2 * i + 1] - output[size - 2 * i + 1]);
	h2r = ((long)output[2 * i + 1] + output[size - 2 * i + 1]);
	h2i = -1 * ((long)output[2 * i] - output[size - 2 * i]);

	output[2 * i] = ((h1r << 15) + h2r * twtble[index * 2 * i] + h2i * twtble[index * 2 * i + 1] + 32768) >> 16;
	output[2 * i + 1] = ((h1i << 15) + h2i * twtble[index * 2 * i] - h2r * twtble[index * 2 * i + 1] + 32768) >> 16;
	output[(size - 2 * i)] = ((h1r << 15) - h2i * twtble[index * 2 * i + 1] - h2r * twtble[index * 2 * i] + 32768) >> 16;
	output[(size - 2 * i + 1)] = (-(h1i << 15) + h2i * twtble[index * 2 * i] - h2r * twtble[index * 2 * i + 1] + 32768) >> 16;
    }

    /* Calculating 1st and 2nd element of the output */
    output[0] = (h1r = output[0]) + output[1];
    output[1] = h1r - output[1];

    return EDSP_OK;             /* Return success */

}
