/*
** FILE NAME:   fir.c
**
** PROJECT ID:  94-0040
**
** TITLE:       Fir
**
** AUTHOR:      Ensigma Ltd.
**
** DESCRIPTION: Perform finite impulse response (FIR) filtering
**
**              int Fir(output, input, no_samples, coeff,
**                      no_coeffs, res_shift, workspace)
** PARAMETERS:
**              short output[]        : output samples y.
**              const short input[]   : input samples x.
**              long no_samples       : number of samples N to be filtered.
**              const short coeff[]   : array of filter coefficients h.
**              long no_coeffs        : number of coefficients K
**              int res_shift         : right shift applied to each output.
**              short *workspace      : variable containing pointer to filter
**                                      memory.
**
**
** RETURNS:     EDSP_OK               : success
**              EDSP_BAD_ARG          : no_samples <  1
**                                      no_coeffs  <= 2
**                                      res_shift  <  0
**                                      res_shift  >  27
**
** MODIFICATION HISTORY:
**              1.0  15-9-95    Initial revision
*/

#include <math.h>
#include <ensigdsp.h>

int
Fir(short output[],
    const short input[],
    long no_samples,
    const short coeff[],
    long no_coeffs,
    int res_shift,
    short *workspace)
{
    /* Variable declaration */
    int i, j, next, inc;
    long newfirst;
    double temp, temp1;

    /* Error Checking */
    if ((no_samples < 1) || (no_coeffs <= 2) || (res_shift < 0) || (res_shift > 27))
	return EDSP_BAD_ARG;

    /* Get the location of the buffer pointer */
    inc = ((((long)workspace[0] << 16) + (long)workspace[1]) - (long)workspace) / 2;

    next = inc;

    /* Copy the input into workspace */
    workspace[inc] = input[0];

    for (j = 0; j < no_samples; j++)	/* Main Loop */
    {
	i = 0;			/* counter initialisation */
	temp = 0;		/* accumulator initialisation */

	while (i < no_coeffs)
	{
	    /* This workspace is anti clock wise circular buffer */

	    if (inc - i < 2)	/* check for valid buffer position */
		inc = no_coeffs + i + 1;	/* move to end of buffer */

	    temp += (double)coeff[i] * workspace[inc - i];
	    i++;		/* while counter increment */
	}

	/* Pointer to target of next input data copy */
	if (next > no_coeffs)
	    next = 2;
	else
	    next++;

	inc = next;		/* Buffer pointer */

	if (j < (no_samples - 1))
	    workspace[next] = input[j + 1];

	/* Rounding and shifting */
	temp1 = floor(temp / (1L << res_shift) + 0.5);

	/* Saturation */
	if (temp1 > 32767)
	    temp1 = 32767;
	else if (temp1 < -32768)
	    temp1 = -32768;

	output[j] = (short)temp1;
    }

    /* store the buffer pointer */
    newfirst = (long)&workspace[next];

    workspace[0] = newfirst >> 16;
    workspace[1] = (newfirst & 0xffff);

    return EDSP_OK;		/* Success */
}
