/*
** FILE NAME:   fir1.c
**
** PROJECT ID:  94-0040
**
** TITLE:       Fir1
**
** AUTHOR:      Ensigma Ltd.
**
** DESCRIPTION: Perform finite impulse response (FIR) filtering for a single
**              input sample
**
**              int Fir1(output, input, coeff,
**                      no_coeffs, res_shift, workspace)
** PARAMETERS:
**              short output[]        : output sample y.
**              short input           : input sample x.
**              const short coeff[]   : array of filter coefficients h.
**              long no_coeffs        : number of coefficients K
**              int res_shift         : right shift applied to each output.
**              short *workspace      : variable containing pointer to filter
**                                      memory.
**
**
** RETURNS:     EDSP_OK               : success
**              EDSP_BAD_ARG          : no_coeffs  <= 2
**                                      res_shift  <  0
**                                      res_shift  >  27
**
** MODIFICATION HISTORY:
**              1.0  15-9-95    Initial revision
*/

#include <math.h>
#include <ensigdsp.h>

int
Fir1(short *output,
     short input,
     const short coeff[],
     long no_coeffs,
     int res_shift,
     short *workspace)
{

    /* Variable Declaration */
    int i, inc, inc_store;
    long newfirst;
    double temp;

    /* Error Checking */
    if ((no_coeffs <= 2) || (res_shift < 0) || (res_shift > 27))
	return EDSP_BAD_ARG;

    /* get the location of the buffer pointer */
    inc = ((((long)workspace[0] << 16) + (long)workspace[1]) - (long)workspace) / 2;
    inc_store = inc;

    /* Copy the input into workspace */

    workspace[inc] = input;

    /* Variable initialisation */
    i = 0;
    temp = 0;

    while (i < no_coeffs)
    {
	/* workspace is anti clockwise circular buffer */

	if ((inc - i) < 2)	/* Check for end of buffer */
	    inc = no_coeffs + i + 1;	/* move to end of buffer */

	temp += (double)coeff[i] * workspace[inc - i];
	i++;			/* while counter increment */
    }

    if (inc_store == (no_coeffs + 1))	/* workspace pointer increment */
	newfirst = (long)&workspace[2];
    else
	newfirst = (long)&workspace[inc_store + 1];

    /* store location of the buffer pointer */
    workspace[0] = newfirst >> 16;
    workspace[1] = (newfirst & 0xffff);

    /* Rounding and Right Shifting */
    temp = floor(temp / (1L << res_shift) + 0.5);

    /* Saturation */
    if (temp > 32767)
	temp = 32767;
    else if (temp < -32768)
	temp = -32768;

    *output = temp;

    return EDSP_OK;		/* Success */
}
