/*
** FILE NAME: testfft.c
**
** PROJECT ID: 94-0040
**
** TITLE: File containing example test functions
**
** AUTHOR: Ensigma Ltd.
**
** DESCRIPTION: This file contains the tests for the inverse FFT functions
**              IfftComplex()
**              IfftReal()
**              IfftInComplex() and
**              IfftInReal().
**
** MODIFICATION HISTORY:
**
**              1.00    15-Sep-95       Eddy
**
*/
#include <math.h>
#include <stdlib.h>
#include <stdio.h>

#include <ensigdsp.h>
#include "testhead.h"

#define PI 3.141592654
#define SCALE_BASE      0x7ff0

#define MIN_CFFT_SIZE   4
#define MIN_RFFT_SIZE   8

#define MAX_IFFT_SIZE  128      /* maximum size to be tested */

static int TightCompare(void *newsig, void *ref, long n, double newshrink);

static long scales[] =
{EFFTNOSCALE, EFFTMIDSCALE, EFFTALLSCALE};
#define NO_SCALES       (sizeof(scales)/sizeof(scales[0]))

extern int debug;               /* set to non-zero if debugging */

/*
** FUNCTION:    TestIfft
**
** AUTHOR:      Ensigma Ltd.
**
** DESCRIPTION: This function tests and demonstrates the use of the DSPLib
**              functions:
**                 IfftComplex
**                 IfftInComplex
**                 IfftReal
**                 IfftInReal
**              The function computes a forward FFT onrandom noise, followed
**              by an inverse FFT. It then verifies that the resulting
**              signal is the same as the original.
**
** PARAMETERS:  rcode       string buffer for status diagnostics
**              n_samp      FFT size to be tested
**              ifft_scale  scaling control parameter
**              ip_peak     The amount by which to scale the data for each
**                          transform. This should be chosen to avoid the
**                          possibility of numerical overflow.
**
** RETURNS:     Integer indicating the number of failures in the function, or
**              -1      if valid arguments were rejected by some function.
**
*/
static int
TestOneIfft(char *rcode, long n_samp, long ifft_scale, long ip_peak)
{
    int j;
    short ip[MAX_IFFT_SIZE * 2];/* input array */
    short ref[MAX_IFFT_SIZE * 2];       /* reference array, a copy of the input
					 * array */
    short op[MAX_IFFT_SIZE * 2];/* normal output array */
    int fail = 0;

    /*
     *  Generate a frame of noise, and copy to the reference array
     */
    for (j = 0; j < 2 * n_samp; j++)
	ip[j] = ip_peak * (2.0 * rand() / 0x7fff - 1.0) / sqrt(2.0);
    for (j = 0; j < 2 * n_samp; j++)
	ref[j] = ip[j];

    /*
     * Test the complex routines
     */
    sprintf(rcode, "Complex IFFT test (size = %ld)\n", n_samp);
    if (FftInComplex(ip, n_samp, ~ifft_scale) != EDSP_OK)
	return -1;
    if (IfftComplex(op, ip, n_samp, ifft_scale) != EDSP_OK)
	return -1;
    if (IfftInComplex(ip, n_samp, ifft_scale) != EDSP_OK)
	return -1;
    fail += TightCompare(op, ref, 2 * n_samp, 1.0);
    fail += TightCompare(ip, ref, 2 * n_samp, 1.0);
    if (fail)
	return fail;

    /*
     * Reload the input array
     */
    for (j = 0; j < 2 * n_samp; j++)
	ip[j] = ref[j];

    /*
     * Test the real transforms.
     */
    sprintf(rcode, "Real IFFT test (size = %ld)\n", n_samp);
    if (n_samp >= MIN_RFFT_SIZE)
    {
	if (FftInReal(ip, n_samp, ~ifft_scale) != EDSP_OK)
	    return -1;
	if (IfftReal(op, ip, n_samp, ifft_scale) != EDSP_OK)
	    return -1;
	if (IfftInReal(ip, n_samp, ifft_scale) != EDSP_OK)
	    return -1;
	fail += TightCompare(op, ref, n_samp, 1.0);
	fail += TightCompare(ip, ref, n_samp, 1.0);
    }

    return (fail);
}

/*
** FUNCTION:    TestIfft
**
** AUTHOR:      Ensigma Ltd.
**
** DESCRIPTION: Test the inverse FFT with inputs sized 2^n, from 2 to
**              MAX_IFFT_SIZE. Non power of two inputs sizes are also tested
**              (no FFT processing should be performed in this case).
*/
int
TestIfft(char *rcode)
{
    int j, scale_idx, size, result = 0;
    int ippeak;
    short data[MIN_CFFT_SIZE * 2 + MIN_RFFT_SIZE];
    long ifftscale;

    for (scale_idx = 0; scale_idx < NO_SCALES; scale_idx++)
    {
	ifftscale = scales[scale_idx];

	/* Check small transforms */
	sprintf(rcode, "Testing argument checking of complex FFT's\n");
	for (size = -1; size < MIN_CFFT_SIZE; size++)
	{
	    if (debug)
		printf("Testing inverse complex transform size %d\n", size);
	    if ((IfftInComplex(data, size, ifftscale) != EDSP_BAD_ARG) ||
		(IfftComplex(data, data, size, ifftscale) != EDSP_BAD_ARG))
		return 1;
	}
	sprintf(rcode, "Testing argument checking of real FFT's\n");
	for (size = -1; size < MIN_RFFT_SIZE; size++)
	{
	    if (debug)
		printf("Testing inverse real transform size %d\n", size);
	    if ((IfftInReal(data, size, ifftscale) != EDSP_BAD_ARG) ||
		(IfftReal(data, data, size, ifftscale) != EDSP_BAD_ARG))
		return 1;
	}

	/*
	 * Due to the nature of the tests, large transforms include
	 * increasing amounts of quantisation noise for certain scales[]
	 * values, so a maximum transform size of 32 is tested.
	 */
	for (size = MIN_CFFT_SIZE; size <= MAX_IFFT_SIZE; size *= 2)
	{
	    /* Check all non powers of two */
	    sprintf(rcode, "Testing rejection of size %d\n", size);
	    for (j = size + 1; j < 2 * size; j++)
	    {
		if (debug & 2)
		    printf("Testing IfftInComplex call for size = %d\n", j);
		if (IfftInComplex(data, j, ifftscale) != EDSP_BAD_ARG)
		    return -1;
		if (debug & 2)
		    printf("Testing IfftComplex call for size = %d\n", j);
		if (IfftComplex(data, data, j, ifftscale) != EDSP_BAD_ARG)
		    return -1;
		if (debug & 2)
		    printf("Testing IfftInReal call for size = %d\n", j);
		if (IfftInReal(data, j, ifftscale) != EDSP_BAD_ARG)
		    return -1;
		if (debug & 2)
		    printf("Testing IfftReal call for size = %d\n", j);
		if (IfftReal(data, data, j, ifftscale) != EDSP_BAD_ARG)
		    return -1;
	    }
	    printf("Testing inverse transforms size %d\n", size);
	    ippeak = SCALE_BASE / size;
	    result += TestOneIfft(rcode, size, ifftscale, ippeak);
	    printf("Error count now %d\n", result);
	    if (result)
		return result;
	}
    }

    sprintf(rcode, "TestIfft returned %d\n", result);
    return result;
}

/*
** FUNCTION:    sqr
**
** DESCRIPTION: Squares its input
*/
static double
sqr(double d)
{
    return d * d;
}

/*
** FUNCTION:    TightCompare
**
** AUTHOR:      Ensigma Ltd.
**
** DESCRIPTION: This function compares the two input arrays using a tight
**              tolerance.
**
** PARAMETERS:  newsig - processed signal
**              ref - reference signal
**              n   - number of elements to be compared
**
** RETURNS:     0 if arrays the same, 1 if arrays different by more than
**              the tolerance.
**
*/
static int
TightCompare(void *newsig, void *ref, long n, double newshrink)
{
#define T_TOLERANCE     0.08   
/*
** This must be fairly high to be able to test large inverse transforms. 
** 0.082 was chosen so that the ansi function library passes for inverse
** transforms of 64 points
*/

    short *snewsig, *sref;
    int i;
    double rmserror = 0.0, signal = 0.0;

    snewsig = (short *)newsig;
    sref = (short *)ref;

    if (debug > 255)
    {
	printf("newshrink = %g\n", newshrink);
	for (i = 0; i < n; i++)
	    printf("%d\t%d\n", snewsig[i], sref[i]);
    }

    for (i = 0; i < n; i++)
    {
	signal += sqr(*sref);
	rmserror += sqr(*snewsig++ / (double)newshrink - *sref++);
    }
    if (debug > 1)
	printf("signal = %g\trmserror = %g\n", signal, rmserror);
    signal = sqrt(signal / n);
    rmserror = sqrt(rmserror / n);
    if (debug)
	printf("Error is %g\n", rmserror / signal);
    return (rmserror / signal >= T_TOLERANCE);
#undef T_TOLERANCE
}
